@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 #
 #--------------------------------------------------------------------------------
-#www2sms.py v1.0.1
+#www2sms.py v1.0.2
 #Copyright Bjoern Olausson
 #--------------------------------------------------------------------------------
 #This program is free software; you can redistribute it and/or modify
@@ -24,6 +24,7 @@
 #
 
 import os, errno, pwd, sys, math, sqlite3, time, calendar, evolution, re
+import ctypes, uuid, glib, gobject
 from PyQt4 import QtCore
 from PyQt4.QtGui import QApplication, QMainWindow, QDialog, QPushButton, QMessageBox
 from www2sms_gui_main import Ui_MainWindow
@@ -334,6 +335,7 @@ class MainWindow(QMainWindow, Ui_MainWindow):
 	    	#Check if a receiver number was provided
 		# Read the number to which we send the SMS
 		NUMBER = unicode(self.ui.lineEdit_Number.text())
+		ORGNUM = NUMBER
 		# Check if an number was entered at all
 	    	if not NUMBER:
 		    	QMessageBox.warning(self, 'www2sms', "Please enter an number")
@@ -352,8 +354,8 @@ class MainWindow(QMainWindow, Ui_MainWindow):
 				return
 		# Check if there is some text to send at all
 		# Read the text we want to send
-		SMSTEXT = unicode(self.ui.plainTextEdit_SMS.toPlainText())
-		if not SMSTEXT:
+		self.SMSTEXT = unicode(self.ui.plainTextEdit_SMS.toPlainText())
+		if not self.SMSTEXT:
 			QMessageBox.information(self, 'www2sms', "Please enter some text")
 			return
 		self.ui.pushButton_Send.setText("Sending...")
@@ -368,23 +370,28 @@ class MainWindow(QMainWindow, Ui_MainWindow):
 			# Read the password
 			PASSWORD = unicode(AccSet.ui.lineEdit_SetupPassword.text())
 			# send the SMS
-			STATUS, MSG = provider.sendSMS(self, app, NUMBER, SMSTEXT, LOGIN, PASSWORD)
+			STATUS, MSG = provider.sendSMS(self, app, NUMBER, self.SMSTEXT, LOGIN, PASSWORD)
 			self.Stat.close()
 			if STATUS == True:
-				self.on_send_success(MSG)
+				self.on_send_success(MSG, ORGNUM)
 			else:
-				self.on_send_failed(MSG)
+				self.on_send_failed(MSG, ORGNUM)
 		self.ui.pushButton_Send.setEnabled(True)
 		self.ui.pushButton_Send.setText("Send")
 
 	# Method which is called from providers class when the sms was successfully send
-	def on_send_success(self,MSG):
+	def on_send_success(self,MSG,NUM):
 		QMessageBox.information(self, 'www2sms', MSG)
 		self.ui.lineEdit_Number.clear()
 		self.ui.plainTextEdit_SMS.clear()
+		print "insertion started"
+		# grp is last 7 (or fewer) digits/whatever
+		grp = NUM[-7:]
+		sendername = "" # do we really need to specify this ourselves? :(
+		put_to_sent_sms(sendername, NUM, 0, grp, self.SMSTEXT)
 
 	# Method which is called from providers class when the sms failed to send
-	def on_send_failed(self,MSG):
+	def on_send_failed(self,MSG,NUM):
 		QMessageBox.warning(self, 'www2sms', MSG)
 
 def DialogStatusMessage(self, statusmessage):
@@ -427,6 +434,83 @@ def mk_db():
 	c.close()
 	conn.close()
 
+# Information taken from:
+# http://docs.python.org/library/uuid.html#uuid.uuid4
+# http://docs.python.org/library/ctypes.html
+# http://www.dalkescientific.com/writings/NBN/ctypes.html
+# Class constructed from: http://maemo.gitorious.org/maemo-rtcom/rtcom-eventlogger/blobs/master/rtcom-eventlogger/event.h
+# credits to gri for doing the hard work with this
+# http://talk.maemo.org/showpost.php?p=661038&postcount=177
+class RTComElEvent(ctypes.Structure):
+	_fields_ = [("_mask", ctypes.c_int),
+	("fld_id", ctypes.c_int),
+	("fld_service_id", ctypes.c_int),
+	("fld_event_type_id", ctypes.c_int),
+	("fld_storage_time", ctypes.c_long), # time_t
+	("fld_start_time", ctypes.c_long), # time_t
+	("fld_end_time", ctypes.c_long), # time_t
+	("fld_is_read", ctypes.c_int), # Boolean is int
+	("fld_flags", ctypes.c_int),
+	("fld_bytes_sent", ctypes.c_int),
+	("fld_bytes_received", ctypes.c_int),
+	("fld_local_uid", ctypes.c_char_p),
+	("fld_local_name", ctypes.c_char_p),
+	("fld_remote_uid", ctypes.c_char_p),
+	("fld_remote_name", ctypes.c_char_p),
+	("fld_remote_ebook_uid", ctypes.c_char_p),
+	("fld_channel", ctypes.c_char_p),
+	("fld_free_text", ctypes.c_char_p),
+	("fld_group_uid", ctypes.c_char_p),
+	("fld_service", ctypes.c_char_p),
+	("fld_event_type", ctypes.c_char_p),
+	("fld_additional_text", ctypes.c_char_p),
+	("fld_icon_name", ctypes.c_char_p),
+	("fld_pango_markup", ctypes.c_char_p)]
+
+def put_to_sent_sms(name, number, uniqueid, groupid, message):
+	try:
+	    rtcom = ctypes.CDLL('librtcom-eventlogger.so.1') # PR 1.2
+	except:
+	    rtcom = ctypes.CDLL('librtcom-eventlogger.so.0') # PR 1.1
+	el = ctypes.cast(rtcom.rtcom_el_new(), ctypes.c_void_p)
+	event = ctypes.cast(rtcom.rtcom_el_event_new(), ctypes.POINTER(RTComElEvent))[0]
+	event._mask       |= 1 << 18
+	event.fld_service  = 'RTCOM_EL_SERVICE_SMS'
+	event._mask       |= 1 << 19
+	event.fld_event_type = 'RTCOM_EL_EVENTTYPE_SMS_OUTBOUND'
+	event._mask       |= 1 << 4
+	event.fld_start_time = int(time.time())
+	event._mask       |= 1 << 5
+	event.fld_end_time = int(time.time())
+	event._mask       |= 1 << 6
+	event.fld_is_read = 1 # TRUE
+	event._mask       |= 1 << 10
+	event.fld_remote_ebook_uid = uniqueid
+	event._mask       |= 1 << 11
+	event.fld_local_uid = 'ring/tel/ring'
+	event._mask       |= 1 << 12
+	event.fld_local_name = '<SelfHandle>'
+	event._mask       |= 1 << 13
+	event.fld_remote_uid = number
+	event._mask       |= 1 << 14
+	event.fld_remote_name = name
+	event._mask       |= 1 << 17
+	event.fld_group_uid = groupid
+	event._mask       |= 1 << 16
+	event.fld_free_text = message
+
+	#print "adding event..."
+	rtcom.rtcom_el_add_event.argtypes = [el, ctypes.c_void_p, ctypes.c_void_p]
+	eventId = rtcom.rtcom_el_add_event(el, ctypes.byref(event), None)
+	#print "added event: %s" % eventId
+	# the follwing block doesnt seem to be needed for SMS
+	# Add the message that it shows up in conversations
+	#rtcom.rtcom_el_add_header.argtypes = [el, ctypes.c_int, ctypes.c_char_p, ctypes.c_char_p, ctypes.c_void_p]
+	rtcom.rtcom_el_add_header(el, eventId, 'message-token', str(uuid.uuid4()), None)
+	rtcom.rtcom_el_event_free(ctypes.byref(event))
+	#glib.g_object_unref(el)
+
+
 # Connect to DB
 mk_db()
 conn = sqlite3.connect('www2sms.db')
